/**

 * \file main.cpp
 * \brief Egyszerű demo az SDL-lel kombinált Widget osztályok kipróbálására.
 * \mainpage
 *  \par Demonstrációs cél:
 *  - callback technika bemutatása
 *  - eseményvezérlés
 *  - a grafikus felhasználói felület eseményvezérelt működése
 *  - egyszerű SDL grafika
 *  - újrafelhasználható elemek
 *  \par Specifikáció:
 *  Eseményekre reagáló alakzatok (körök) felrakása a képernyőre. Vezérlő gombok (töröl, kilép) kialakítása.
 *  \par Események:
 *   Egérmozgatás, kattintás, húzás (drag)
 *  \par Magyarázatok:
 *  Az SDL 1.x nem biztosít külön ablakokat. Minden egy ablakban van, ezért a Widget-ek is
 *  bárhol megjelenhetnek ebben. A Widget-ek felrakási sorrendben "takarják" egymást, de tényleges
 *  felülettel csak a körök rendelkeznek. A Bulletinboard (Bboard) háttere "átlátszó". Ezért elsőnek az
 *  ActiveBg nevű Widget-et tesszük fel. Ezután a nyomógombokat, majd a Bulletinboard-ot.
 *  Az ActiveBg "elkapja" a hozzá eljutó egérgomb eseményt és meghívja a hozzárendelt
 *  függvényt.
 *  Mivel a nyomógombok belső állapotokat változtatnak a nyomógombokhoz rendelt funkciókat (kivéve quit)
 *  a Bbpanel osztály valósítja meg, ami az állapotot tárolja. Ennek azonban csak egy belépési pontja van, a
 *  függvényhívás operátor, ezért a funkcióválasztást egy int paraméter segíti.
 *  \par Érdekesség:
 *  Osztály tagfüggvényét önállóan nem lehet függvéynpointerrel meghívni, hiszen a híváshoz egy \em this is kell.
 *  Ezért nem lehet a quit függvényhez hasonlóan egy osztály tagfüggvényét meghívni.
 *  \par UML
 *  \image html bboard.png "Az UMLet programmal rajzolt osztálydiagram" height=5
 *  Ez nem mindenben egyezik meg a Doxygen által generált ábrával. Az ábráról lemaradt az ActiveBg template, ami
 *  A Button sablonhoz hasonló, csak hazudósabb, és nincs grafikus reprezentációja.
 *
 *  http://www.umlet.com/
 *
 *
 */

#include <iostream>
#include "appshell.h"
#include "bboard.h"
#include "button.h"
#include "activebg.h"
#include "memtrace.h"

/// segédfüggvény az SDL inicializálásához
/// @return renderer pointer vagy 0
SDL_Renderer* init() {
    if (SDL_Init(SDL_INIT_EVERYTHING) < 0) {
        return 0;
    }
    SDL_Window *sdlWindow;
    SDL_Renderer *sdlRenderer;
    if (SDL_CreateWindowAndRenderer(640, 480, SDL_WINDOW_RESIZABLE, &sdlWindow, &sdlRenderer) < 0) {
        return 0;
    }
    SDL_RenderClear(sdlRenderer);

    return sdlRenderer;
}

/// Bulletinboard kezelő panelje
class Bbpanel {
    Bboard& bb; ///< ref. a táblára
    int r;      ///< kirajzolandó kör sugara
    Color c;    ///< kor színe
    void chcolor() {
        c.r += 50; c.g -= 50; c.b -= 10;
    }
public:
    static const int RIGTH_ON_BG = 0;
    static const int ADD_CIRCLE = 1;
    static const int CLEAR = 3;
    // Konstruktor átveszi a táblát, és inicializálja a sugarat és színt
    Bbpanel(Bboard& bb) :bb(bb), r(200), c(WHITE) {}

    // ez az általános callback fv.
    void operator()(const SDL_Event& ev, int fn) {
        if (fn == RIGTH_ON_BG) {
            // jobb gomb a háttéren
            bb.add(new Circle(Point(ev.button.x, ev.button.y), 20, c));
            chcolor();
        } else if (fn == ADD_CIRCLE) {
            // kör hozzáadása gomb
            bb.add(new Circle(Point(350, 350), r, c));
            chcolor();
            r -= 10;
            if (r <= 0) {
                r = 200; c = WHITE;
            }
        } else if (fn == CLEAR) {
            // clear gomb
            bb.clear();
        }
    }
};

/// Kilépés, üzen saját magának
void quit(const SDL_Event&, int) {
    SDL_Event ev;
    ev.type = SDL_QUIT;
    SDL_PushEvent(&ev);
}

int main(int argc, char** argv) {

    SDL_Renderer* renderer = init();
    if (renderer == NULL) {
        std::cerr << "Nem sikerult az inicializálás " << SDL_GetError() << std::endl;
        return 1;
    }

    // Application shell létrehozása
    AppShell app(renderer);
    Bboard* bb = new Bboard;    // Bulletinboard
    // Bbpanel olyan osztály, ami segít a gombokat kezelni.
    Bbpanel panel(*bb);         // ismernie kell a Bulletinboard-ot

    // Először feltesszük az aktív hátteret (ez kap utoljára eseményt)
    app.add(new ActiveBg<Bbpanel>(panel, Bbpanel::RIGTH_ON_BG));   // jobb gomb a háttéren

    app.add(bb);                // rátesszük a bulletinboard-ot
    // Feltesszük a gombokat
    app.add(new Button<Bbpanel>(Point(100, 60), 40, panel, Bbpanel::ADD_CIRCLE, GREEN));  // add circle
    app.add(new Button<Bbpanel>(Point(300, 60), 30, panel, Bbpanel::CLEAR, RED));    // clear BB
    app.add(new Button<>(Point(500, 60), 20, quit));                    // quit

    // eseményhurok
    app.mainLoop();
    SDL_Quit();
    return 0;
}
